# -*- coding: utf-8 -*-

import xbmc
import xbmcgui
from config import logger, ADDON_NAME, RUTOR_URL, USE_PROXY, EXCHANGE_TORRENT_INFO
import json
from database import Database, normalize_torrent_info_url
from urlparse import urljoin
from exchange_torrent_info import ExchangeTorrentInfo


def send_download_torrent_info_notification():
    xbmc.executeJSONRPC(json.dumps({
        "jsonrpc": "2.0",
        "method": "JSONRPC.NotifyAll",
        "params": {
            "sender": ADDON_NAME,
            "message": "DownloadTorrentInfo",
        },
        "id": 1
    }))


class EStopException(Exception):
    pass


class SDownloadTorrentInfo(xbmc.Monitor):
    def __init__(self):
        super(SDownloadTorrentInfo, self).__init__()
        self.start_work = False

    def onNotification(self, sender, method, data):
        logger.debug("SDownloadTorrentInfo onNotification %s, %s, %s" % (sender, method, data))
        if sender == ADDON_NAME:
            if method == 'Other.DownloadTorrentInfo':
                self.start_work = True

    def start(self):
        try:
            while not self.abortRequested():
                if self.start_work:
                    self.start_work = False
                    self.download_torrent_info()
                else:
                    self.waitForAbort(5)
        except EStopException:
            pass
        logger.debug('SDownloadTorrentInfo stoped')

    def download_torrent_info(self):
        from rutor import RuTor
        logger.debug('do download_torrent_info')
        db = Database()
        inactive_torrent_info = db.torrent_info_inactive()
        if inactive_torrent_info is None:
            return
        inactive_torrent_info = inactive_torrent_info[-9:]
        s_total = len(inactive_torrent_info)
        s_current = 0.0
        rutor = RuTor(RUTOR_URL)
        pd = xbmcgui.DialogProgressBG()
        pd.create('Загрузка информации о торрентах')
        for t_path, status, data in inactive_torrent_info:
            t_url = urljoin(RUTOR_URL, t_path)
            if self.abortRequested():
                raise EStopException()
            try:
                info = rutor.get_info(t_url)
            except Exception, e:
                logger.exception(t_url, exc_info=True)
                db.torrent_info_del(t_url)
                continue
            db.torrent_info_update(t_url, info)
            ExchangeTorrentInfo().set_info(normalize_torrent_info_url(t_url), info)
            s_current += 1
            title = ' '
            if 'title' in info['kodi_info']:
                title = info['kodi_info']['title']
            else:
                logger.debug('Not found title in %s' % t_url)
            pd.update(int(s_current/s_total*100), title, t_path)
            if self.waitForAbort(2):  # Если слишком часто обращаться к рутору, подумает что досим, ждем 2 сек
                raise EStopException()
        pd.close()
        del pd


if __name__ == '__main__':
    service = SDownloadTorrentInfo()
    service.start()

